//
// c l o c k . c
//
// GBA clock!
//
// June 25, 2003 by Wataru Nishida (http://www.skyfree.org)
//

#include "gba.h"
#include "screen.h"
#include "console.h"
#include "printf.h"

// Global variables which will be updated in INT handler

int tmr1_cnt = 0;	// Timer 1 overflow-counter
int key_cnt = 0;	// Keyboard overflow-counter

// Interrupt handler

void int_handler(void) {
  int flag;

  register(INT_IME) = INT_IME_OFF;
  flag = register(INT_IF);
  if (flag & INT_KEY) {
    tmr1_cnt = 0;	// Reset timer counter
    key_cnt++;		// Increment key counter
   }
  if (flag & INT_TMR1) {
    tmr1_cnt++;		// Increment counter
   }
  register(INT_IF) = flag;
  register(INT_IME) = INT_IME_ON;
 }

int main() {
  int hour, min, sec;

  // Initialize screen

  init_screen();

  // Reset timer counters

  register(TMR_COUNT0) = 0;	// Timer INT frequency = 1 / 2^24 CPU cycles
  register(TMR_COUNT1) = 0xFF00;

  // Initialize timers

  register(TMR_CTRL0) = TMR_ENABLE + TMR_1CLOCK;
  register(TMR_CTRL1) = TMR_ENABLE + TMR_CASCADE + TMR_INTREN;

  // Initialize key interruption

  register(KEY_INTCNT) = KEY_INTAND + KEY_INTEN + KEY_L + KEY_R;

  // Activate interruption!

  register(INT_IE) = INT_TMR1 | INT_KEY;
  register(INT_IME) = INT_IME_ON;

  // Loop forever

  selectpage(BG_TEXT1);
  while (1) {
    locate(8, 7);
    printf(" Timer 0: %04x", register(TMR_COUNT0));
    locate(8, 9);
    printf(" Timer 1: %04x", register(TMR_COUNT1));
    locate(8, 11);
    hour = tmr1_cnt / 3600;
    min = (tmr1_cnt - hour * 3600) / 60;
    sec = tmr1_cnt % 60;
    printf("Clock: %02d:%02d:%02d", hour, min, sec);
    locate(8, 13);
    printf("  Resets: %d", key_cnt);
   }
 }
