//
// i n t . c
//
// Interruption demonstration program
//
// June 25, 2003 by Wataru Nishida (http://www.skyfree.org)
//

#include "gba.h"
#include "screen.h"
#include "console.h"
#include "printf.h"

// Global variables which will be updated in INT handler

int tmr0_cnt = 0;	// Timer 0 overflow-counter
int tmr1_cnt = 0;	// Timer 1 overflow-counter

void int_handler(void) {
  int flag;

  register(INT_IME) = INT_IME_OFF;
  flag = register(INT_IF);
  if (flag & INT_TMR0) {
    tmr0_cnt++;		// Increment counter
   }
  if (flag & INT_TMR1) {
    tmr1_cnt++;		// Increment counter
   }
  register(INT_IF) = flag;
  register(INT_IME) = INT_IME_ON;
 }

int main() {
  // Initialize screen

  init_screen();

  // Reset timer counters

  register(TMR_COUNT0) = 0;	// Timer INT frequency = 1 / 2^24 CPU cycles
  register(TMR_COUNT1) = 0xFF00;

  // Initialize timers

  register(TMR_CTRL0) = TMR_ENABLE + TMR_1CLOCK + TMR_INTREN;
  register(TMR_CTRL1) = TMR_ENABLE + TMR_CASCADE + TMR_INTREN;

  // Activate interruption!

  register(INT_IE) = INT_TMR0 | INT_TMR1;
  register(INT_IME) = INT_IME_ON;

  // Loop forever

  selectpage(BG_TEXT1);
  while (1) {
    locate(6, 7);
    printf("   Timer 0: %04x", register(TMR_COUNT0));
    locate(6, 9);
    printf("   Timer 1: %04x", register(TMR_COUNT1));
    locate(6, 11);
    printf("Overflow 0: %d", tmr0_cnt);
    locate(6, 13);
    printf("Overflow 1: %d", tmr1_cnt);
   }
 }
